#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#This script determines all the differentially regulated probes for each group by group comparison
#in an ArrayExpress experiment using Affymetrix chips. 

#load the libraries
library(Biobase)
library(limma)
library("ArrayExpress")
library(affy)

#create header line for output contain summary infomation about each group by group comparison
summary=c("ID","Factor 1","Factor 2","category","n1","n2")


#Filename is the ArrayExpress dataset name, ie. E-MEXP-3167
filename<-"E-MEXP-3167"

#load and normalize ArrayExpress experiment
AEset<-ArrayExpress(filename,save=FALSE)
AEset
Norm_AEset<-rma(AEset)
cols<-colnames(pData(Norm_AEset))

#Determine how factors
groups<-""
for(itr in 1:length(cols))
{
    temp<-grep("FACTOR",toupper(cols[itr]))
    if (length(temp)>0)
    {
        groups<-c(groups,itr)
    }
}

#Compare each array by different variable types: ie. Disease.state, gender, time, etc...
itr2<-0
for(itr in 2:length(groups)) 
{        
	if (groups[itr]!="sample" && groups[itr]!="description") 
	{
		itr2<-itr2+1
		
		sorted_list=sort(pData(Norm_AEset)[as.integer(groups[itr])][,1])

		f<-factor				
		f<-factor(sorted_list)
		design <- model.matrix(~0+f)
		colnames(design)<-(colnames(design))
		design_cols<-colnames(design)
		#Compare each group to one another if have more than 2 groups
		if (length(design_cols)>1) {
			
			#iterate thru each permutation
			for (design_col_itr in 1:(length(design_cols)-1)) {
				design_col_itr2<-design_col_itr+1
				while (design_col_itr2<length(design_cols)+1)
				{
					print(paste(design_col_itr,"-",design_col_itr2,":",length(design_cols)))

					#make factor vector and remove unused groups
					sml <-""
					flag_G0<-0
					flag_G1<-0
					label1<-""
					label2<-""
					
					for (states_itr in 1:length(sorted_list)) 
					{
						
						if (paste("f",pData(Norm_AEset)[as.integer(groups[itr])][,1][states_itr],sep="")==design_cols[design_col_itr] ) {
							sml<-c(sml,"G0")
							flag_G0<-flag_G0+1
							label1<-pData(Norm_AEset)[as.integer(groups[itr])][,1][states_itr]
						} else if ( paste("f",pData(Norm_AEset)[as.integer(groups[itr])][,1][states_itr],sep="")==design_cols[design_col_itr2] ) {
							sml<-c(sml,"G1")
							flag_G1<-flag_G1+1
							label2<-pData(Norm_AEset)[as.integer(groups[itr])][,1][states_itr]
						} else {
							sml<-c(sml,"X")									
						}
					}
					
					#Analyze if >2 samples
					if ( flag_G0>1 && flag_G1>1) {
						sml<-sml[2:length(sml)]
						print (sml)
	
						# set up the data and proceed with analysis
						# eliminate samples marked as "X"
						sel <- which(sml != "X")
						sml <- sml[sel]
						temp_eset <- Norm_AEset[ ,sel]
									
						# set up the data and proceed with analysis
						fl <- as.factor(sml)
								
						temp_eset$description <- fl
														
						design <- model.matrix(~ description + 0, temp_eset)
						colnames(design) <- levels(fl)
					
						fit <- lmFit(temp_eset, design)
						cont.matrix <- makeContrasts(G1-G0, levels=design)							
						fit2 <- contrasts.fit(fit, cont.matrix)
						fit2 <- eBayes(fit2)
						tT <- topTable(fit2, adjust="fdr", sort.by="B", number=nrow(fit2))
                                                  
						tT2<- subset(tT, adj.P.Val < 0.05)
						file_ID=paste(filename,"_",paste(groups[itr]),'_',design_col_itr,'_',design_col_itr2,sep="")
						write.table(tT2, file=paste(file_ID,sep=""), row.names=F, sep="\t")

						temp_r=c(file_ID,paste(label1) ,paste(label2),cols[   as.integer(groups[itr])   ],flag_G0,flag_G1)
						summary<-rbind(summary,temp_r)
                           
					}
					design_col_itr2<-design_col_itr2+1
				}
			}
		}
	}
}

#write summary file
write.table(summary, file=paste(filename,"_summary.txt",sep=""), sep="\t")

